@@ -344,6 +344,7 @@ void fm_config_load_from_file(FmConfig* cfg, const char* name)
     cfg->modules_blacklist = NULL;
     cfg->system_modules_blacklist = NULL;
     _cfg_monitor_free(cfg);
+    g_free(cfg->_cfg_name);
     if(G_LIKELY(!name))
         name = "libfm/libfm.conf";
     else
@@ -362,17 +363,18 @@ void fm_config_load_from_file(FmConfig* cfg, const char* name)
 
     cfg->_cfg_name = g_strdup(name);
     dirs = g_get_system_config_dirs();
-    for(dir=dirs;*dir;++dir)
+    /* bug SF #887: first dir in XDG_CONFIG_DIRS is the most relevant
+       so we shoult process the list in reverse order */
+    dir = dirs;
+    while (*dir)
+        ++dir;
+    while (dir-- != dirs)
     {
         path = g_build_filename(*dir, name, NULL);
         if(g_key_file_load_from_file(kf, path, 0, NULL))
             fm_config_load_from_key_file(cfg, kf);
         g_free(path);
     }
-    path = g_build_filename(SYSCONF_DIR, name, NULL);
-    if(g_key_file_load_from_file(kf, path, 0, NULL))
-        fm_config_load_from_key_file(cfg, kf);
-    g_free(path);
     /* we got all system blacklists, save them and get user's one */
     cfg->system_modules_blacklist = cfg->modules_blacklist;
     cfg->modules_blacklist = NULL;
