@@ -501,7 +501,10 @@ void fm_file_info_set_from_g_file_data(FmFileInfo *fi, GFile *gf, GFileInfo *inf
 
     g_return_if_fail(fi->path);
 
-    _fm_path_set_display_name(fi->path, g_file_info_get_display_name(inf));
+    tmp = g_file_info_get_edit_name(inf);
+    if (!tmp)
+        tmp = g_file_info_get_display_name(inf);
+    _fm_path_set_display_name(fi->path, tmp);
 
     fi->size = g_file_info_get_size(inf);
 
@@ -585,7 +588,8 @@ void fm_file_info_set_from_g_file_data(FmFileInfo *fi, GFile *gf, GFileInfo *inf
                 fi->mime_type = fm_mime_type_from_file_name(fi->target);
         }
 
-        if(G_UNLIKELY(!fi->mime_type))
+        /* if the mime-type is not determined or is unknown */
+        if(G_UNLIKELY(!fi->mime_type || g_content_type_is_unknown(fm_mime_type_get_type(fi->mime_type))))
         {
             /* FIXME: is this appropriate? */
             if(type == G_FILE_TYPE_SHORTCUT)
@@ -651,7 +655,8 @@ _file_is_symlink:
     fi->ctime = g_file_info_get_attribute_uint64(inf, G_FILE_ATTRIBUTE_TIME_CHANGED);
     fi->hidden = g_file_info_get_is_hidden(inf);
     fi->backup = g_file_info_get_is_backup(inf);
-    fi->name_is_changeable = fi->icon_is_changeable = fi->hidden_is_changeable = FALSE;
+    fi->name_is_changeable = TRUE; /* GVFS tends to ignore this attribute */
+    fi->icon_is_changeable = fi->hidden_is_changeable = FALSE;
     if (g_file_info_has_attribute(inf, G_FILE_ATTRIBUTE_ACCESS_CAN_RENAME))
         fi->name_is_changeable = g_file_info_get_attribute_boolean(inf, G_FILE_ATTRIBUTE_ACCESS_CAN_RENAME);
     if (G_UNLIKELY(gf == NULL))
