@@ -252,6 +252,12 @@ FmTerminal* fm_terminal_dup_default(GError **error)
     return term;
 }
 
+static void child_setup(gpointer user_data)
+{
+    /* Move child to grandparent group so it will not die with parent */
+    setpgid(0, (pid_t)user_data);
+}
+
 /**
  * fm_terminal_launch
  * @dir: (allow-none): a directory to launch
@@ -327,7 +333,8 @@ gboolean fm_terminal_launch(const gchar *dir, GError **error)
         *env = g_strdup_printf ("PWD=%s", dir);
     }
 #endif
-    ret = g_spawn_async(dir, argv, envp, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, error);
+    ret = g_spawn_async(dir, argv, envp, G_SPAWN_SEARCH_PATH,
+                        child_setup, (gpointer)getpgid(getppid()), NULL, error);
     g_strfreev(argv);
     g_strfreev(envp);
     return ret;
