@@ -783,20 +783,6 @@ static void on_column_auto_adjust(GtkMenuItem* menu_item, GtkTreeViewColumn* col
 static gboolean on_column_button_press_event(GtkWidget *button,
                                              GdkEventButton *event,
                                              GtkTreeViewColumn* col)
-{
-    if(event->button == 1)
-    {
-        GtkWidget* view = gtk_tree_view_column_get_tree_view(col);
-        GtkWidget* fv = gtk_widget_get_parent(view);
-        FmFolderViewColumnInfo* info = g_object_get_qdata(G_OBJECT(col), fm_qdata_id);
-        g_return_val_if_fail(FM_IS_STANDARD_VIEW(fv), FALSE);
-        return !fm_folder_model_col_is_sortable(FM_STANDARD_VIEW(fv)->model, info->col_id);
-    }
-    return FALSE;
-}
-
-static gboolean on_column_button_released_event(GtkWidget *button, GdkEventButton *event,
-                                        GtkTreeViewColumn* col)
 {
     if(event->button == 3)
     {
@@ -897,6 +883,20 @@ static gboolean on_column_button_released_event(GtkWidget *button, GdkEventButto
     return FALSE;
 }
 
+static gboolean on_column_button_released_event(GtkWidget *button, GdkEventButton *event,
+                                        GtkTreeViewColumn* col)
+{
+    if(event->button == 1)
+    {
+        GtkWidget* view = gtk_tree_view_column_get_tree_view(col);
+        GtkWidget* fv = gtk_widget_get_parent(view);
+        FmFolderViewColumnInfo* info = g_object_get_qdata(G_OBJECT(col), fm_qdata_id);
+        g_return_val_if_fail(FM_IS_STANDARD_VIEW(fv), FALSE);
+        return !fm_folder_model_col_is_sortable(FM_STANDARD_VIEW(fv)->model, info->col_id);
+    }
+    return FALSE;
+}
+
 static GtkTreeViewColumn* create_list_view_column(FmStandardView* fv,
                                                   FmFolderViewColumnInfo *set)
 {
@@ -1791,11 +1791,12 @@ static void _fm_standard_view_scroll_to_path(FmFolderView* fv, FmPath *path, gbo
     GtkTreeIter it;
     GtkTreePath *tp;
 
-    if (!FM_IS_STANDARD_VIEW(fv))
+    if (!FM_IS_STANDARD_VIEW(fv) || path == NULL)
         return;
     view = (FmStandardView*)fv;
-    fm_folder_model_find_iter_by_filename(view->model, &it,
-                                          fm_path_get_basename(path));
+    if (!fm_folder_model_find_iter_by_filename(view->model, &it,
+                                               fm_path_get_basename(path)))
+        return;
     tp = gtk_tree_model_get_path(GTK_TREE_MODEL(view->model), &it);
     if (tp == NULL) /* invalid child */
         return;
